@@ -1,12 +1,12 @@
 import os
-from pathlib import Path
 import time
+from pathlib import Path
 from timeit import default_timer as timer
-import typer
 from typing import Dict, List, Optional
 
-from modelator import __version__
-from modelator import ModelResult, const_values
+import typer
+
+from modelator import ModelResult, __version__, const_values
 from modelator.cli.model_config_file import load_config_file
 from modelator.cli.model_file import ModelFile
 from modelator.itf import ITF
@@ -93,7 +93,7 @@ def load(
         try:
             config = load_config_file(config_path)
         except FileNotFoundError:
-            print(f"ERROR: config file not found")
+            print("ERROR: config file not found")
             raise typer.Exit(code=4)
 
     ModelFile.save(model, config, config_path)
@@ -120,7 +120,7 @@ def reload():
         try:
             config = load_config_file(config_path)
         except FileNotFoundError:
-            print(f"ERROR: config file not found")
+            print("ERROR: config file not found")
             raise typer.Exit(code=4)
 
     ModelFile.save(model, config, config_path)
@@ -147,6 +147,54 @@ def typecheck():
         raise typer.Exit(code=6)
 
 
+@app.command(
+    context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
+)
+def simulate(
+    ctx: typer.Context,
+    model_path: Optional[str] = typer.Option(None, help="Path to the TLA+ model file."),
+    config_path: Optional[str] = typer.Option(
+        None, help="Path to TOML file with model and model checker configurations."
+    ),
+    init: Optional[str] = typer.Option(None, help="Model's init predicate."),
+    next: Optional[str] = typer.Option(None, help="Model's next predicate."),
+    constants: Optional[str] = typer.Option(
+        None,
+        help="Comma-separated list of constant definitions in the format 'name=value' (overwrites config file).",
+    ),
+    length: Optional[int] = typer.Option(
+        default=10, help="Number of steps in each simulation"
+    ),
+    max_trace: Optional[int] = typer.Option(
+        default=5, help="Number of simulated traces to generate."
+    ),
+    traces_dir: Optional[str] = typer.Option(
+        None,
+        help="Path to store generated trace files (overwrites config file).",
+    ),
+):
+    """
+    Generate execution traces by simulating the model evolution.
+    """
+    model, config = _load_model_with_arguments(
+        "simulate",
+        None,
+        model_path,
+        config_path,
+        init,
+        next,
+        constants,
+        traces_dir,
+        ctx.args,
+    )
+
+    config["params"]["save_runs"] = True
+    config["params"]["length"] = length
+    config["params"]["max_run"] = max_trace
+
+    _run_checker("simulate", model, config)
+
+
 @app.command(
     context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
 )
@@ -279,7 +327,7 @@ def _load_config_and_merge_arguments(
     try:
         config = load_config_file(config_path)
     except FileNotFoundError:
-        print(f"ERROR: config file not found")
+        print("ERROR: config file not found")
         raise typer.Exit(code=4)
 
     # Overwrite configuration with passed arguments
@@ -294,12 +342,12 @@ def _load_config_and_merge_arguments(
         config_from_arguments[properties_config_name] = properties
     if traces_dir:
         config_from_arguments["traces_dir"] = traces_dir
-    config |= config_from_arguments
+    config = {**config, **config_from_arguments}
 
     # Update model-cheker arguments. Note that `extra_args` may contain any
     # field name, even some not supported in the toml configuration file.
     if extra_args:
-        config["params"] |= extra_args
+        config["params"] = {**config["params"], **extra_args}
         config_from_arguments["params"] = extra_args
 
     return config, config_from_arguments
@@ -322,7 +370,7 @@ def _load_model_with_arguments(
     """
     if mode == "check":
         properties_config_name = "invariants"
-    elif mode == "sample":
+    elif mode == "sample" or mode == "simulate":
         properties_config_name = "examples"
     else:
         raise ValueError("Unknown checker mode")
@@ -354,8 +402,8 @@ def _load_model_with_arguments(
     if not model:
         model, saved_config, _ = ModelFile.load(LOG_LEVEL)
         if saved_config:
-            config |= saved_config
-            config |= config_from_arguments
+            config = {**config, **saved_config}
+            config = {**config, **config_from_arguments}
 
     if not model:
         print(
@@ -365,7 +413,7 @@ def _load_model_with_arguments(
         )
         raise typer.Exit(code=1)
 
-    if not config[properties_config_name]:
+    if (mode == "sample" or mode == "check") and not config[properties_config_name]:
         print(
             "ERROR: could not find properties to check; either:\n"
             "- load a configuration together with a model "
@@ -399,6 +447,10 @@ def _run_checker(mode, model, config):
         handler = model.sample
         action = "Sampling"
         properties_config_name = "examples"
+    elif mode == "simulate":
+        handler = model.simulate
+        action = "Simulating"
+        properties_config_name = "examples"
     else:
         raise ValueError("Unknown checker mode")
 
@@ -406,8 +458,8 @@ def _run_checker(mode, model, config):
     print("{} {}... ".format(action, ", ".join(config[properties_config_name])))
     result: ModelResult = handler(
         config[properties_config_name],
-        constants=config["constants"],
         checker_params=config["params"],
+        constants=config["constants"],
         traces_dir=config["traces_dir"],
     )
 
@@ -449,7 +501,7 @@ def reset():
     Removes any loaded model.
     """
     if ModelFile.clean():
-        print(f"Model file removed")
+        print("Model file removed")
 
 
 @app.command()
@@ -470,10 +522,10 @@ app_apalache = typer.Typer(
 app.add_typer(app_apalache, name="apalache")
 
 
-@app_apalache.command()
-def info(
+@app_apalache.command(name="info")
+def apalache_info(
     version: Optional[str] = typer.Argument(
-        const_values.DEFAULT_APALACHE_VERSION, help=f"Apalache's version."
+        const_values.DEFAULT_APALACHE_VERSION, help="Apalache's version."
     ),
 ):
     """
@@ -489,13 +541,13 @@ def info(
         existing_version = apalache_jar.apalache_jar_version(jar_path)
         print(f"Apalache JAR file exists and its version is {existing_version}")
     else:
-        print(f"Apalache JAR file not found")
+        print("Apalache JAR file not found")
 
 
 @app_apalache.command()
 def get(
     version: Optional[str] = typer.Argument(
-        const_values.DEFAULT_APALACHE_VERSION, help=f"Apalache's version."
+        const_values.DEFAULT_APALACHE_VERSION, help="Apalache's version."
     ),
 ):
     """
