@@ -19,8 +19,9 @@ def write_trace_files_to(apalache_result: Dict, traces_dir: str) -> List[str]:
     # create directory if it does not exist
     Path(traces_dir).mkdir(parents=True, exist_ok=True)
 
+    itfs_filenames_pattern = re.compile(r"\d\.itf\.json$")
     itfs_filenames = [
-        f for f in apalache_result["files"].keys() if f.endswith(".itf.json")
+        f for f in apalache_result["files"].keys() if itfs_filenames_pattern.search(f)
     ]
     itfs_filenames.sort()
 
@@ -39,13 +40,29 @@ def write_trace_files_to(apalache_result: Dict, traces_dir: str) -> List[str]:
     return trace_paths
 
 
-def extract_counterexample(files: Dict[str, str]):
-    cex_tla_content = files[APALACHE_DEFAULTS["result_violation_tla_file"]]
+def extract_simulations(files: Dict[str, str], num_simulations: int = 1):
+    itf_tests = []
+    for i in range(num_simulations):
+        file_name = "example" + str(i) + ".itf.json"
+        test_content = json.loads(files[file_name])["states"]
+        itf_tests.append(test_content)
+
+    return itf_tests
+
+
+def extract_counterexample(
+    files: Dict[str, str], trace_name=APALACHE_DEFAULTS["trace_name"]
+):
+
+    tla_file_name = trace_name + ".tla"
+    cex_tla_content = files[tla_file_name]
+
     violated_invariant = extract_line_with(
         APALACHE_STDOUT["INVARIANT_VIOLATION"], cex_tla_content
     )
 
-    itf_file_content = files[APALACHE_DEFAULTS["result_violation_itf_file"]]
+    itf_file_name = trace_name + ".itf.json"
+    itf_file_content = files[itf_file_name]
     counterexample = json.loads(itf_file_content)["states"]
 
     return (violated_invariant, counterexample)
@@ -95,7 +112,7 @@ def extract_typecheck_error(parser_output: str):
         if reportActive is True:
             # find error reports which point to exact locations
             match_exact_loc = re.search(
-                "\[(?P<fileName>\w+\.tla):(?P<lineNumber>\d+):.+\]: (?P<info>.+) E@.+",
+                r"\[(?P<fileName>\w+\.tla):(?P<lineNumber>\d+):.+\]: (?P<info>.+) E@.+",
                 line,
             )
             if match_exact_loc is not None:
