@@ -1,8 +1,9 @@
+import logging
 import os
 import re
 from typing import Dict, Optional
+
 from .. import const_values
-import logging
 
 
 def create_logger(logger_name, loglevel):
@@ -43,9 +44,6 @@ def wrap_command(
     json_command = {}
     json_command["args"] = {}
 
-    if args is not None and cmd != const_values.CHECK_CMD:
-        assert const_values.CONFIG not in args
-
     if checker == const_values.APALACHE:
         json_command["args"]["cmd"] = cmd
 
