@@ -1,11 +1,8 @@
 import os
-from operator import neg
 from typing import Dict, List, Tuple
 
 import modelator_py.util.tla as tla_parsing
 
-from modelator import const_values
-
 
 def get_auxiliary_tla_files(model_name: str) -> Dict[str, str]:
 
